/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import at.tuwien.ifs.feature.extraction.audio.data.FeatureExtractionOptions;
import at.tuwien.ifs.feature.extraction.audio.data.FeatureVectorData;
import at.tuwien.ifs.feature.extraction.audio.input.SimpleFileListReader;
import at.tuwien.ifs.feature.extraction.audio.util.LogHandler;
import at.tuwien.ifs.feature.extraction.audio.util.MyLogger;

/**
 * Copyright Vienna University of Technology
 * 
 * @author Thomas Lidy
 * @author Jakob Frank
 * @author Rudolf Mayer
 * @version $Id: MainGUI.java 217 2011-05-07 11:04:55Z mayer $
 */
public class MainGUI extends JFrame implements LogHandler {

    public enum Status {
        SET_UP, EXTRACTING, DONE
    }

    private Status state;

    private static final long serialVersionUID = 3L;

    private JPanel mainPanel = null;

    private JPanel inputPanel = null;

    private JRadioButton rdoSelDirectory = null;

    private JRadioButton rdoSelFilelist = null;

    private JPanel pnlFileOpen = null;

    private JCheckBox chkFiletypeWAV = null;

    private JCheckBox chkFiletypeAU = null;

    private JCheckBox chkFiletypeMP3 = null;

    private JTextField txtInput = null;

    private JPanel pnlFileTypes = null;

    private JLabel jLabel = null;

    private JPanel pnlFileSave;

    private JPanel pnlOutput = null;

    private JLabel labelBaseName = null;

    private JTextField txtBasename = null;

    private JTextField txtOutput = null;

    private JLabel labelOutputDirectory = null;

    private JPanel pnlOptions = null;

    private JCheckBox chkRP = null;

    private JCheckBox chkRH = null;

    private JCheckBox chkSSD = null;

    private JLabel labelFeatures = null;

    private JLabel labelThreads = null;

    private JSpinner spnThreads = null;

    private SpinnerNumberModel noThreads = null;

    private JProgressBar overallProgress = null;

    private JPanel pnlProgress = null;

    private JLabel lblOverallProgress = null;

    private JLabel lblThreadProgress = null;

    private JPanel pnlButtons = null;

    private JButton btnGo = null;

    private JButton btnCancel = null;

    private JCheckBox chkZVec = null;

    private JFileChooser fileChooser = null;

    private MultipleFileExtractor multipleFileExtractor = null;

    private JPanel pnlErrWarn = null;

    private JScrollPane scpErrWarn = null;

    private JTextArea txtErrWarn = null;

    private JLabel labelOutputFormats = null;

    private JCheckBox chkVec = null;

    private JCheckBox chkArff = null;

    private JCheckBox chkZArff = null;

    private JCheckBox chkArffVecLabel = null;

    private JCheckBox chkOsiris = null;

    private JCheckBox chkXml = null;

    private JPanel pnlFeatures = null;

    public void setStatus(Status state) {
        this.state = state;

        switch (state) {
            case SET_UP:
                getInputPanel().setEnabled(true);
                getPnlOutput().setEnabled(true);
                getPnlOptions().setEnabled(true);
                getPnlProgress().setVisible(false);

                getBtnGo().setEnabled(true);
                getBtnCancel().setText("Close");
                break;
            case EXTRACTING:
                getInputPanel().setEnabled(false);
                getPnlOutput().setEnabled(false);
                getPnlOptions().setEnabled(false);
                getPnlProgress().setVisible(true);

                getBtnGo().setEnabled(false);
                getBtnCancel().setText("Cancel");
                break;
            default: // Finished
                getInputPanel().setEnabled(true);
                getPnlOutput().setEnabled(true);
                getPnlOptions().setEnabled(true);
                getPnlProgress().setVisible(true);

                getBtnGo().setEnabled(true);
                getBtnCancel().setText("Close");
                break;
        }
    }

    /**
     * This method initializes mainPanel
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getMainPanel() {
        if (mainPanel == null) {
            GridBagConstraints gridBagConstraints23 = new GridBagConstraints();
            gridBagConstraints23.fill = GridBagConstraints.BOTH;
            gridBagConstraints23.gridx = 0;
            gridBagConstraints23.weighty = 1.0;
            gridBagConstraints23.weightx = 0.0;
            gridBagConstraints23.gridy = 4;
            GridBagConstraints gridBagConstraints25 = new GridBagConstraints();
            gridBagConstraints25.gridx = 0;
            gridBagConstraints25.fill = GridBagConstraints.BOTH;
            gridBagConstraints25.anchor = GridBagConstraints.EAST;
            gridBagConstraints25.gridy = 5;
            GridBagConstraints gridBagConstraints22 = new GridBagConstraints();
            gridBagConstraints22.gridx = 0;
            gridBagConstraints22.fill = GridBagConstraints.BOTH;
            gridBagConstraints22.gridy = 3;
            GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
            gridBagConstraints14.gridx = 0;
            gridBagConstraints14.fill = GridBagConstraints.BOTH;
            gridBagConstraints14.gridy = 2;
            GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
            gridBagConstraints9.gridx = 0;
            gridBagConstraints9.fill = GridBagConstraints.BOTH;
            gridBagConstraints9.gridy = 1;
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.fill = GridBagConstraints.BOTH;
            mainPanel = new JPanel();
            mainPanel.setLayout(new GridBagLayout());
            mainPanel.add(getInputPanel(), gridBagConstraints1);
            mainPanel.add(getPnlOutput(), gridBagConstraints9);
            mainPanel.add(getPnlOptions(), gridBagConstraints14);
            mainPanel.add(getPnlProgress(), gridBagConstraints22);
            mainPanel.add(getPnlErrWarn(), gridBagConstraints23);
            mainPanel.add(getPnlButtons(), gridBagConstraints25);
        }
        return mainPanel;
    }

    /**
     * This method initializes inputPanel
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getInputPanel() {
        if (inputPanel == null) {
            GridBagConstraints gridBagConstraints110 = new GridBagConstraints();
            gridBagConstraints110.anchor = GridBagConstraints.WEST;
            GridBagConstraints gridBagConstraints61 = new GridBagConstraints();
            gridBagConstraints61.gridx = 0;
            gridBagConstraints61.anchor = GridBagConstraints.EAST;
            gridBagConstraints61.gridwidth = 0;
            gridBagConstraints61.weightx = 0.0;
            gridBagConstraints61.fill = GridBagConstraints.NONE;
            gridBagConstraints61.gridy = 2;
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
            gridBagConstraints2.gridwidth = 0;
            gridBagConstraints2.insets = new Insets(1, 1, 1, 1);
            gridBagConstraints2.weightx = 1.0;
            gridBagConstraints2.gridy = 1;
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 1;
            gridBagConstraints.insets = new Insets(2, 1, 1, 1);
            gridBagConstraints.anchor = GridBagConstraints.WEST;
            gridBagConstraints.gridy = 0;
            inputPanel = new JPanel();
            inputPanel.setBorder(BorderFactory.createTitledBorder("Input"));
            inputPanel.setLayout(new GridBagLayout());
            inputPanel.add(getRdoSelDirectory(), gridBagConstraints110);
            inputPanel.add(getRdoSelFilelist(), gridBagConstraints);
            inputPanel.add(getPnlFileOpen(), gridBagConstraints2);
            inputPanel.add(getPnlFileTypes(), gridBagConstraints61);
            ButtonGroup btgInputType = new ButtonGroup();
            btgInputType.add(getRdoSelDirectory());
            btgInputType.add(getRdoSelFilelist());

            inputPanel.addPropertyChangeListener("enabled", new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    boolean newVal = evt.getNewValue().equals(Boolean.TRUE);
                    getRdoSelDirectory().setEnabled(newVal);
                    getRdoSelFilelist().setEnabled(newVal);
                    getTxtInput().setEnabled(newVal);
                    getPnlFileTypes().setEnabled(newVal);
                }
            });
        }
        return inputPanel;
    }

    /**
     * This method initializes rdoSelDirectory
     * 
     * @return javax.swing.JRadioButton
     */
    private JRadioButton getRdoSelDirectory() {
        if (rdoSelDirectory == null) {
            rdoSelDirectory = new JRadioButton();
            rdoSelDirectory.setText("select directory");
            rdoSelDirectory.setSelected(true);
        }
        return rdoSelDirectory;
    }

    /**
     * This method initializes rdoSelFilelist
     * 
     * @return javax.swing.JRadioButton
     */
    private JRadioButton getRdoSelFilelist() {
        if (rdoSelFilelist == null) {
            rdoSelFilelist = new JRadioButton();
            rdoSelFilelist.setText("select filelist");
        }
        return rdoSelFilelist;
    }

    /**
     * This method initializes pnlFileSave
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlFileSave() {
        if (pnlFileSave == null) {
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            gridBagConstraints7.fill = GridBagConstraints.BOTH;
            GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
            gridBagConstraints6.fill = GridBagConstraints.BOTH;
            gridBagConstraints6.weightx = 1.0;
            pnlFileSave = new JPanel();
            pnlFileSave.setLayout(new GridBagLayout());
            pnlFileSave.add(getTxtOutput(), gridBagConstraints6);
            final JButton btnFileChooser = new JButton();
            btnFileChooser.setMargin(new Insets(0, 0, 0, 0));
            btnFileChooser.setText("...");
            btnFileChooser.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    if (fileChooser.showSaveDialog(MainGUI.this) == JFileChooser.APPROVE_OPTION) {
                        getTxtOutput().setText(fileChooser.getSelectedFile().getAbsolutePath());
                    }
                }
            });
            pnlFileSave.add(btnFileChooser, gridBagConstraints7);
            getTxtOutput().addPropertyChangeListener("enabled", new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    btnFileChooser.setEnabled(evt.getNewValue().equals(Boolean.TRUE));
                }
            });

        }
        return pnlFileSave;
    }

    /**
     * This method initializes pnlFileOpen
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlFileOpen() {
        if (pnlFileOpen == null) {
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            gridBagConstraints7.fill = GridBagConstraints.BOTH;
            GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
            gridBagConstraints6.fill = GridBagConstraints.BOTH;
            gridBagConstraints6.weightx = 1.0;
            pnlFileOpen = new JPanel();
            pnlFileOpen.setLayout(new GridBagLayout());
            pnlFileOpen.add(getTxtInput(), gridBagConstraints6);
            final JButton btnFileChooser = new JButton();
            btnFileChooser.setMargin(new Insets(0, 0, 0, 0));
            btnFileChooser.setText("...");
            btnFileChooser.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    File current = new File(getTxtInput().getText());
                    if (current.exists() && current.isDirectory()) {
                        fileChooser.setCurrentDirectory(current);
                    }
                    if (getRdoSelDirectory().isSelected()) {
                        fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    } else {
                        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
                    }
                    if (fileChooser.showOpenDialog(MainGUI.this) == JFileChooser.APPROVE_OPTION) {
                        getTxtInput().setText(fileChooser.getSelectedFile().getAbsolutePath());
                    }
                }
            });
            pnlFileOpen.add(btnFileChooser, gridBagConstraints7);
            getTxtInput().addPropertyChangeListener("enabled", new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    btnFileChooser.setEnabled(evt.getNewValue().equals(Boolean.TRUE));
                }
            });
        }
        return pnlFileOpen;
    }

    /**
     * This method initializes chkFiletypeWAV
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkFiletypeWAV() {
        if (chkFiletypeWAV == null) {
            chkFiletypeWAV = new JCheckBox();
            chkFiletypeWAV.setText("wav");
            chkFiletypeWAV.setSelected(true);
        }
        return chkFiletypeWAV;
    }

    /**
     * This method initializes chkFiletypeAU
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkFiletypeAU() {
        if (chkFiletypeAU == null) {
            chkFiletypeAU = new JCheckBox();
            chkFiletypeAU.setText("au");
            chkFiletypeAU.setSelected(true);
        }
        return chkFiletypeAU;
    }

    /**
     * This method initializes chkFiletypeMP3
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkFiletypeMP3() {
        if (chkFiletypeMP3 == null) {
            chkFiletypeMP3 = new JCheckBox();
            chkFiletypeMP3.setText("mp3");
            chkFiletypeMP3.setSelected(true);
        }
        return chkFiletypeMP3;
    }

    /**
     * This method initializes txtInput
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getTxtInput() {
        if (txtInput == null) {
            txtInput = new JTextField();
        }
        return txtInput;
    }

    /**
     * This method initializes txtOutput
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getTxtOutput() {
        if (txtOutput == null) {
            txtOutput = new JTextField();
        }
        return txtOutput;
    }

    /**
     * This method initializes pnlFileTypes
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlFileTypes() {
        if (pnlFileTypes == null) {
            GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
            gridBagConstraints8.gridx = 0;
            gridBagConstraints8.gridy = 0;
            jLabel = new JLabel();
            jLabel.setText("Extensions:");
            GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
            gridBagConstraints5.gridx = 3;
            gridBagConstraints5.gridy = -1;
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.gridx = 2;
            gridBagConstraints4.gridy = -1;
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.gridx = 1;
            gridBagConstraints3.insets = new Insets(0, 0, 0, 0);
            gridBagConstraints3.gridy = -1;
            pnlFileTypes = new JPanel();
            pnlFileTypes.setLayout(new GridBagLayout());
            pnlFileTypes.add(getChkFiletypeWAV(), gridBagConstraints3);
            pnlFileTypes.add(getChkFiletypeAU(), gridBagConstraints4);
            pnlFileTypes.add(getChkFiletypeMP3(), gridBagConstraints5);
            pnlFileTypes.add(jLabel, gridBagConstraints8);

            pnlFileTypes.addPropertyChangeListener("enabled", new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    boolean newVal = evt.getNewValue().equals(Boolean.TRUE);
                    jLabel.setEnabled(newVal);
                    getChkFiletypeAU().setEnabled(newVal);
                    getChkFiletypeWAV().setEnabled(newVal);
                    getChkFiletypeMP3().setEnabled(newVal);
                }
            });
        }
        return pnlFileTypes;
    }

    /**
     * This method initializes pnlOutput
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlOutput() {
        if (pnlOutput == null) {
            GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
            gridBagConstraints13.gridwidth = 0;
            gridBagConstraints13.anchor = GridBagConstraints.WEST;
            labelOutputDirectory = new JLabel();
            labelOutputDirectory.setText("Output Directory:");
            GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
            gridBagConstraints12.fill = GridBagConstraints.BOTH;
            gridBagConstraints12.gridwidth = 0;
            gridBagConstraints12.gridy = 1;
            GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
            gridBagConstraints11.gridx = 0;
            gridBagConstraints11.gridy = 2;
            GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
            gridBagConstraints10.fill = GridBagConstraints.BOTH;
            gridBagConstraints10.gridy = 2;
            gridBagConstraints10.weightx = 1.0;
            gridBagConstraints10.insets = new Insets(2, 2, 0, 0);
            gridBagConstraints10.gridx = 1;
            labelBaseName = new JLabel();
            labelBaseName.setText("Basename:");
            pnlOutput = new JPanel();
            pnlOutput.setBorder(BorderFactory.createTitledBorder("Output"));
            pnlOutput.setLayout(new GridBagLayout());
            pnlOutput.add(labelOutputDirectory, gridBagConstraints13);
            pnlOutput.add(getPnlFileSave(), gridBagConstraints12);
            pnlOutput.add(labelBaseName, gridBagConstraints11);
            pnlOutput.add(getTxtBasename(), gridBagConstraints10);
            pnlOutput.addPropertyChangeListener("enabled", new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    boolean enabled = evt.getNewValue().equals(Boolean.TRUE);
                    labelOutputDirectory.setEnabled(enabled);
                    getTxtBasename().setEnabled(enabled);
                    labelBaseName.setEnabled(enabled);
                    getTxtOutput().setEnabled(enabled);
                }
            });
        }
        return pnlOutput;
    }

    /**
     * This method initializes txtBasename
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getTxtBasename() {
        if (txtBasename == null) {
            txtBasename = new JTextField();
        }
        return txtBasename;
    }

    /**
     * This method initializes pnlOptions
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlOptions() {
        if (pnlOptions == null) {
            GridBagConstraints gridBagConstraints35 = new GridBagConstraints();
            gridBagConstraints35.gridx = 1;
            gridBagConstraints35.gridwidth = 0;
            gridBagConstraints35.anchor = GridBagConstraints.WEST;
            gridBagConstraints35.gridy = 0;
            GridBagConstraints gridBagConstraints34 = new GridBagConstraints();
            gridBagConstraints34.gridx = 1;
            gridBagConstraints34.anchor = GridBagConstraints.WEST;
            gridBagConstraints34.gridwidth = 0;
            gridBagConstraints34.gridy = 7;
            GridBagConstraints gridBagConstraints33 = new GridBagConstraints();
            gridBagConstraints33.gridx = 1;
            gridBagConstraints33.anchor = GridBagConstraints.WEST;
            gridBagConstraints33.gridwidth = 0;
            gridBagConstraints33.gridy = 6;
            GridBagConstraints gridBagConstraints32 = new GridBagConstraints();
            gridBagConstraints32.gridx = 2;
            gridBagConstraints32.gridwidth = 0;
            gridBagConstraints32.anchor = GridBagConstraints.WEST;
            gridBagConstraints32.gridy = 4;
            GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
            gridBagConstraints31.gridx = 1;
            gridBagConstraints31.gridwidth = 1;
            gridBagConstraints31.anchor = GridBagConstraints.WEST;
            gridBagConstraints31.gridy = 4;
            GridBagConstraints gridBagConstraints36 = new GridBagConstraints();
            gridBagConstraints36.gridx = 2;
            gridBagConstraints36.gridwidth = 1;
            gridBagConstraints36.anchor = GridBagConstraints.WEST;
            gridBagConstraints36.gridy = 5;
            GridBagConstraints gridBagConstraints30 = new GridBagConstraints();
            gridBagConstraints30.gridx = 1;
            gridBagConstraints30.gridwidth = 1;
            gridBagConstraints30.anchor = GridBagConstraints.WEST;
            gridBagConstraints30.gridy = 2;
            GridBagConstraints gridBagConstraints24 = new GridBagConstraints();
            gridBagConstraints24.gridx = 0;
            gridBagConstraints24.anchor = GridBagConstraints.EAST;
            gridBagConstraints24.insets = new Insets(0, 0, 0, 2);
            gridBagConstraints24.gridy = 2;
            labelOutputFormats = new JLabel();
            labelOutputFormats.setText("Output Formats:");
            GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
            gridBagConstraints21.gridx = 2;
            gridBagConstraints21.gridwidth = 0;
            gridBagConstraints21.anchor = GridBagConstraints.WEST;
            gridBagConstraints21.ipadx = 0;
            gridBagConstraints21.gridy = 2;
            GridBagConstraints gridBagConstraints20 = new GridBagConstraints();
            gridBagConstraints20.fill = GridBagConstraints.VERTICAL;
            gridBagConstraints20.gridy = 1;
            gridBagConstraints20.weightx = 1.0;
            gridBagConstraints20.gridwidth = 0;
            gridBagConstraints20.anchor = GridBagConstraints.WEST;
            gridBagConstraints20.insets = new Insets(0, 2, 0, 0);
            gridBagConstraints20.gridx = 1;
            GridBagConstraints gridBagConstraints19 = new GridBagConstraints();
            gridBagConstraints19.gridx = 0;
            gridBagConstraints19.anchor = GridBagConstraints.EAST;
            gridBagConstraints19.insets = new Insets(0, 0, 0, 2);
            gridBagConstraints19.gridy = 1;
            labelThreads = new JLabel();
            labelThreads.setText("Parallel Threads:");
            GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
            gridBagConstraints17.gridx = 0;
            gridBagConstraints17.anchor = GridBagConstraints.EAST;
            gridBagConstraints17.insets = new Insets(0, 0, 0, 2);
            gridBagConstraints17.gridy = 0;
            labelFeatures = new JLabel();
            labelFeatures.setText("Features:");
            pnlOptions = new JPanel();
            pnlOptions.setBorder(BorderFactory.createTitledBorder("Options"));
            pnlOptions.setLayout(new GridBagLayout());
            pnlOptions.add(labelFeatures, gridBagConstraints17);
            pnlOptions.add(labelThreads, gridBagConstraints19);
            pnlOptions.add(getSpnThreads(), gridBagConstraints20);
            pnlOptions.add(labelOutputFormats, gridBagConstraints24);
            pnlOptions.add(getChkVec(), gridBagConstraints30);
            pnlOptions.add(getChkZVec(), gridBagConstraints21);
            pnlOptions.add(getChkArff(), gridBagConstraints31);
            pnlOptions.add(getChkZArff(), gridBagConstraints32);
            pnlOptions.add(getChkArffVecLabel(), gridBagConstraints36);
            pnlOptions.add(getChkOsiris(), gridBagConstraints33);
            pnlOptions.add(getChkXml(), gridBagConstraints34);
            pnlOptions.add(getPnlFeatures(), gridBagConstraints35);
            pnlOptions.addPropertyChangeListener("enabled", new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    boolean enabled = evt.getNewValue().equals(Boolean.TRUE);
                    labelFeatures.setEnabled(enabled);
                    getChkRH().setEnabled(enabled);
                    getChkRP().setEnabled(enabled);
                    getChkSSD().setEnabled(enabled);

                    labelOutputFormats.setEnabled(enabled);
                    getChkVec().setEnabled(enabled);
                    getChkZVec().setEnabled(enabled);
                    getChkArff().setEnabled(enabled);
                    getChkZArff().setEnabled(enabled);
                    getChkOsiris().setEnabled(enabled);
                    getChkXml().setEnabled(enabled);
                }
            });
        }
        return pnlOptions;
    }

    /**
     * This method initializes chkRP
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkRP() {
        if (chkRP == null) {
            chkRP = new JCheckBox();
            chkRP.setText("RP");
            chkRP.setSelected(true);
        }
        return chkRP;
    }

    /**
     * This method initializes chkRH
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkRH() {
        if (chkRH == null) {
            chkRH = new JCheckBox();
            chkRH.setText("RH");
            chkRH.setSelected(true);
        }
        return chkRH;
    }

    /**
     * This method initializes chkSSD
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkSSD() {
        if (chkSSD == null) {
            chkSSD = new JCheckBox();
            chkSSD.setText("SSD");
            chkSSD.setSelected(true);
        }
        return chkSSD;
    }

    /**
     * This method initializes spnThreads
     * 
     * @return javax.swing.JSpinner
     */
    private JSpinner getSpnThreads() {
        if (spnThreads == null) {
            spnThreads = new JSpinner();
            spnThreads.setModel(getNoThreads());
            spnThreads.setToolTipText("Number of parallel threads, can be dynamically adjusted.");
        }
        return spnThreads;
    }

    private SpinnerNumberModel getNoThreads() {
        if (noThreads == null) {
            noThreads = new SpinnerNumberModel(1, 1,
                    Runtime.getRuntime().availableProcessors() * 2, 1);
            noThreads.addChangeListener(new ChangeListener() {

                @Override
                public void stateChanged(ChangeEvent e) {
                    if (multipleFileExtractor != null) {
                        multipleFileExtractor.setParallelThreads(noThreads.getNumber().intValue());
                    }
                }
            });

        }
        return noThreads;
    }

    /**
     * This method initializes overallProgress
     * 
     * @return javax.swing.JProgressBar
     */
    JProgressBar getOverallProgress() {
        if (overallProgress == null) {
            overallProgress = new JProgressBar();
            overallProgress.setStringPainted(true);
        }
        return overallProgress;
    }

    /**
     * This method initializes pnlProgress
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlProgress() {
        if (pnlProgress == null) {
            lblThreadProgress = new JLabel();
            lblThreadProgress.setText("Thread-Progress:");
            lblOverallProgress = new JLabel();
            lblOverallProgress.setText("Overall Progress:");
            pnlProgress = new JPanel();
            pnlProgress.setBorder(BorderFactory.createTitledBorder("Progress"));
            BoxLayout boxLayout = new BoxLayout(getPnlProgress(), BoxLayout.Y_AXIS);
            pnlProgress.setLayout(boxLayout);
            pnlProgress.add(lblOverallProgress, null);
            pnlProgress.add(getOverallProgress(), null);
            pnlProgress.add(lblThreadProgress, null);
            pnlProgress.setVisible(true);
        }
        return pnlProgress;
    }

    /**
     * This method initializes pnlButtons
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlButtons() {
        if (pnlButtons == null) {
            GridBagConstraints gridBagConstraints29 = new GridBagConstraints();
            gridBagConstraints29.insets = new Insets(5, 10, 5, 5);
            gridBagConstraints29.gridy = 0;
            gridBagConstraints29.anchor = GridBagConstraints.EAST;
            gridBagConstraints29.fill = GridBagConstraints.NONE;
            gridBagConstraints29.gridx = 1;
            GridBagConstraints gridBagConstraints28 = new GridBagConstraints();
            gridBagConstraints28.gridx = 0;
            gridBagConstraints28.anchor = GridBagConstraints.EAST;
            gridBagConstraints28.gridy = 0;
            GridBagConstraints gridBagConstraints27 = new GridBagConstraints();
            gridBagConstraints27.gridx = 1;
            GridBagConstraints gridBagConstraints26 = new GridBagConstraints();
            gridBagConstraints26.gridx = 0;
            gridBagConstraints26.gridy = 0;
            pnlButtons = new JPanel();
            pnlButtons.setLayout(new GridBagLayout());
            pnlButtons.add(getBtnCancel(), gridBagConstraints28);
            pnlButtons.add(getBtnGo(), gridBagConstraints29);
        }
        return pnlButtons;
    }

    /**
     * This method initializes btnGo
     * 
     * @return javax.swing.JButton
     */
    private JButton getBtnGo() {
        if (btnGo == null) {
            btnGo = new JButton("Start");
            btnGo.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    if (getTxtBasename().getText().trim().length() == 0) {
                        MyLogger.error("No basename given, Stop.");
                        getTxtBasename().requestFocus();
                        return;
                    }
                    if (getTxtOutput().getText().trim().length() == 0) {
                        MyLogger.error("No output dir given, Stop.");
                        getTxtOutput().requestFocus();
                        return;
                    }

                    setStatus(Status.EXTRACTING);
                    final FeatureExtractionOptions opt = getFeatureExtractionOptions();
                    try {
                        final List<FeatureVectorData> list;
                        if (getRdoSelFilelist().isSelected()) {
                            list = new SimpleFileListReader(new File(getTxtInput().getText())).getMetaDataList();
                        } else {
                            ArrayList<String> ext = new ArrayList<String>(3);
                            // "wav", "au", "mp3"
                            if (getChkFiletypeWAV().isSelected()) {
                                ext.add("wav");
                            }
                            if (getChkFiletypeAU().isSelected()) {
                                ext.add("au");
                            }
                            if (getChkFiletypeMP3().isSelected()) {
                                ext.add("mp3");
                            }
                            list = MultipleFileExtractor.createExtractionList(new File(
                                    getTxtInput().getText()), ext.toArray(new String[ext.size()]),
                                    opt);
                        }
                        new Thread() {

                            @Override
                            public void run() {
                                try {
                                    multipleFileExtractor = new MultipleFileExtractor(
                                            noThreads.getNumber().intValue());

                                    multipleFileExtractor.extractListOfFiles(list, opt,
                                            MainGUI.this);
                                } finally {
                                    multipleFileExtractor = null;
                                }
                            };
                        }.start();
                    } catch (IOException e1) {
                        JOptionPane.showMessageDialog(MainGUI.this, e1.getMessage());
                        setStatus(Status.SET_UP);
                    }
                }
            });
        }
        return btnGo;
    }

    /**
     * This method initializes btnCancel
     * 
     * @return javax.swing.JButton
     */
    private JButton getBtnCancel() {
        if (btnCancel == null) {
            btnCancel = new JButton("Cancel");
            btnCancel.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    doExit();
                }
            });
        }
        return btnCancel;
    }

    /**
     * This method initializes chkZVec
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkZVec() {
        if (chkZVec == null) {
            chkZVec = new JCheckBox("SOMLib Vec (gzip)");
            chkZVec.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if (chkZVec.isSelected()) {
                        getChkVec().setSelected(false);
                    }
                }
            });

        }
        return chkZVec;
    }

    /**
     * This method initializes pnlErrWarn
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlErrWarn() {
        if (pnlErrWarn == null) {
            pnlErrWarn = new JPanel();
            pnlErrWarn.setBorder(BorderFactory.createTitledBorder("Errors & Warnings"));
            pnlErrWarn.setLayout(new BorderLayout());
            pnlErrWarn.add(getScpErrWarn(), BorderLayout.CENTER);
            pnlErrWarn.setMinimumSize(new Dimension(0, 150));
            pnlErrWarn.setVisible(false);
        }
        return pnlErrWarn;
    }

    /**
     * This method initializes scpErrWarn
     * 
     * @return javax.swing.JScrollPane
     */
    private JScrollPane getScpErrWarn() {
        if (scpErrWarn == null) {
            scpErrWarn = new JScrollPane();
            scpErrWarn.setViewportView(getTxtErrWarn());
        }
        return scpErrWarn;
    }

    /**
     * This method initializes txtErrWarn
     * 
     * @return javax.swing.JTextArea
     */
    private JTextArea getTxtErrWarn() {
        if (txtErrWarn == null) {
            txtErrWarn = new JTextArea();
            // txtErrWarn.setEnabled(false);
            txtErrWarn.setEditable(false);
            txtErrWarn.setLineWrap(true);
            txtErrWarn.setRows(7);
        }
        return txtErrWarn;
    }

    /**
     * This method initializes chkVec
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkVec() {
        if (chkVec == null) {
            chkVec = new JCheckBox("SOMLib Vec");
            chkVec.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if (chkVec.isSelected()) {
                        getChkZVec().setSelected(false);
                    }
                }
            });
        }
        return chkVec;
    }

    /**
     * This method initializes chkArff
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkArff() {
        if (chkArff == null) {
            chkArff = new JCheckBox("ARFF");
            chkArff.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if (chkArff.isSelected()) {
                        getChkZArff().setSelected(false);
                    }
                    chkArffVecLabel.setEnabled(chkArff.isSelected() || chkZArff.isSelected());
                }
            });

        }
        return chkArff;
    }

    /**
     * This method initializes chkZArff
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkZArff() {
        if (chkZArff == null) {
            chkZArff = new JCheckBox("ARFF (gzip)");
            chkZArff.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if (chkZArff.isSelected()) {
                        getChkArff().setSelected(false);
                    }
                    chkArffVecLabel.setEnabled(chkArff.isSelected() || chkZArff.isSelected());
                }
            });

        }
        return chkZArff;
    }

    /**
     * This method initializes chkArffVecLabel
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkArffVecLabel() {
        if (chkArffVecLabel == null) {
            chkArffVecLabel = new JCheckBox("write vector label");
            chkArffVecLabel.setToolTipText("Whether to write an attribute with the name of the vector in the ARFF file");
            chkArffVecLabel.setEnabled(chkArff.isSelected() || chkZArff.isSelected());
        }
        return chkArffVecLabel;
    }

    /**
     * This method initializes chkOsiris
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkOsiris() {
        if (chkOsiris == null) {
            chkOsiris = new JCheckBox("OSIRIS");
        }
        return chkOsiris;
    }

    /**
     * This method initializes chkXml
     * 
     * @return javax.swing.JCheckBox
     */
    private JCheckBox getChkXml() {
        if (chkXml == null) {
            chkXml = new JCheckBox("XML");
        }
        return chkXml;
    }

    /**
     * This method initializes pnlFeatures
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getPnlFeatures() {
        if (pnlFeatures == null) {
            GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
            gridBagConstraints16.gridx = -1;
            gridBagConstraints16.gridy = -1;
            GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
            gridBagConstraints15.gridx = 2;
            gridBagConstraints15.gridy = 0;
            GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
            gridBagConstraints18.gridx = -1;
            gridBagConstraints18.gridwidth = 1;
            gridBagConstraints18.gridy = -1;
            pnlFeatures = new JPanel();
            pnlFeatures.setLayout(new GridBagLayout());
            pnlFeatures.add(getChkRP(), gridBagConstraints18);
            pnlFeatures.add(getChkRH(), gridBagConstraints15);
            pnlFeatures.add(getChkSSD(), gridBagConstraints16);
        }
        return pnlFeatures;
    }

    public static void main(String[] args) {
        MainGUI mainFrame = new MainGUI();
        mainFrame.setVisible(true);
    }

    public MainGUI() {
        state = Status.SET_UP;
        MyLogger.registerLogHandler(this);

        initialize();
    }

    MainGUI(MultipleFileExtractor extractor) {
        state = Status.EXTRACTING;
        MyLogger.registerLogHandler(this);
        this.multipleFileExtractor = extractor;

        initialize();
    }

    private void initialize() {
        setTitle("AudioFeatureExtractor");
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                doExit();
            }
        });
        setLayout(new GridBagLayout());

        fileChooser = new JFileChooser();

        this.setContentPane(getMainPanel());

        setStatus(state);

        pack();
    }

    public void setFeatureExtractionOptions(FeatureExtractionOptions opt) {
        getChkRH().setSelected(opt.extractRH);
        getChkRP().setSelected(opt.extractRP);
        getChkSSD().setSelected(opt.extractSSD);

        File input = new File(opt.inputBaseDir);
        getTxtInput().setText(input.getPath());
        (input.isDirectory() ? getRdoSelDirectory() : getRdoSelFilelist()).setSelected(true);

        getChkZVec().setSelected(opt.writeSOMLib && opt.enableSOMLibGZip);
        getChkVec().setSelected(opt.writeSOMLib && !opt.enableSOMLibGZip);
        getChkZArff().setSelected(opt.writeWekaARFF && opt.enableArffGZip);
        getChkArff().setSelected(opt.writeWekaARFF && !opt.enableArffGZip);
        getChkOsiris().setSelected(opt.writeOSIRIS);
        getChkXml().setSelected(opt.writeXML);

        File basename = new File(opt.outputBaseFilename);
        getTxtOutput().setText(basename.getParent());
        getTxtBasename().setText(basename.getName());
    }

    public FeatureExtractionOptions getFeatureExtractionOptions() {
        FeatureExtractionOptions opt = new FeatureExtractionOptions();
        opt.extractRH = getChkRH().isSelected();
        opt.extractRP = getChkRP().isSelected();
        opt.extractSSD = getChkSSD().isSelected();

        opt.graphicalEnvironment = true;
        opt.visParentFrame = this;

        File input = new File(getTxtInput().getText());
        opt.inputBaseDir = input.isDirectory() ? input.getAbsolutePath() : input.getParent();
        opt.outputBaseFilename = new File(new File(getTxtOutput().getText()),
                getTxtBasename().getText()).getAbsolutePath();

        opt.writeSOMLib = getChkVec().isSelected() || getChkZVec().isSelected();
        opt.enableSOMLibGZip = getChkZVec().isSelected();
        opt.writeWekaARFF = getChkArff().isSelected() || getChkZArff().isSelected();
        opt.enableArffGZip = getChkZArff().isSelected();
        opt.enableArffVectorLabel = getChkArffVecLabel().isSelected();
        opt.writeOSIRIS = getChkOsiris().isSelected();
        opt.writeXML = getChkXml().isSelected();

        return opt;
    }

    public JProgressBar createThreadProgressBar() {
        JProgressBar bar = new JProgressBar();
        bar.setStringPainted(true);
        getPnlProgress().add(bar);
        lblThreadProgress.setVisible(true);
        pack();
        return bar;
    }

    public void removeThreadProgressBar(JProgressBar progress) {
        getPnlProgress().remove(progress);
        if (getPnlProgress().getComponentCount() <= 3) {
            lblThreadProgress.setVisible(false);
        }
        pack();
    }

    private void doExit() {
        if (state == Status.EXTRACTING) {
            // Cancel
            if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(this,
                    "Extraction is still running. Are you shure you want to stop the process?",
                    "Extraction is running", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)) {

                if (multipleFileExtractor != null) {
                    multipleFileExtractor.cancelExtraction();
                }
                setStatus(Status.SET_UP);
            }
        } else {
            setVisible(false);
            dispose();
            System.exit(0);
        }
    }

    void setThreadNumber(int threads) {
        getNoThreads().setValue(threads);
    }

    int getThreadNumber() {
        return getNoThreads().getNumber().intValue();
    }

    @Override
    public void appendLog(String message, LogGrade grade) {
        switch (grade) {
            case ERROR:
            case WARNING:
                JTextArea txt = getTxtErrWarn();
                synchronized (txt) {
                    txt.append(grade + message + "\n");
                    txt.setCaretPosition(txt.getText().length());
                }
                if (!getPnlErrWarn().isVisible()) {
                    getPnlErrWarn().setVisible(true);
                    this.pack();
                }
                break;
            default:
                // nop
        }
    }

    @Override
    public void flushAllLogs() {
        // nop
    }

    @Override
    public void flushLog() {
        // nop
    }

    @Override
    public void setLogGrade(LogGrade grade) {
        // nop: we only act on Warnings and Errors.
    }

} // @jve:decl-index=0:visual-constraint="10,10"
