/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.data;

import java.awt.Frame;

import at.tuwien.ifs.feature.extraction.audio.vis.ExtractorStatusPanel;

/**
 * Copyright Vienna University of Technology
 * 
 * @author Thomas Lidy
 * @version $Id: FeatureExtractionOptions.java 214 2011-05-07 11:03:32Z mayer $
 */
public class FeatureExtractionOptions {

    public boolean enableSOMLibGZip = false;

    public boolean enableArffGZip = false;

    public boolean enableArffVectorLabel = false;

    public boolean graphicalEnvironment = false;

    public boolean showFeatureVis = false;

    public boolean showIntermediateVis = false;

    public Frame visParentFrame = null;

    public ExtractorStatusPanel progressPanel = null;

    public String inputBaseDir = "";

    public String outputBaseFilename = "project";

    public boolean extractRP = true;

    public boolean extractSSD = true;

    public boolean extractRH = true;

    public int bandLimit = 23;

    public int modAmplLimit = 60;

    public int stepWidth = 3;

    public int skipLeadInFadeOut = 1;

    public boolean writeSOMLib = true;

    public boolean writeXML = false;

    public boolean writeOSIRIS = false;

    public boolean writeWekaARFF = false;

    public boolean combineFeaturesIntoOneFile = false;

    public void disableGraphics() {
        graphicalEnvironment = false;
        showFeatureVis = false;
        showIntermediateVis = false;
        visParentFrame = null;
    }

    public int getNumberOfFeatureSets() {
        if (combineFeaturesIntoOneFile) {
            return 1;
        }

        int n = 0;
        if (extractRP) {
            n++;
        }
        if (extractRH) {
            n++;
        }
        if (extractSSD) {
            n++;
        }

        return n;
    }

    public String[] getFileExtensionsOfFeatureSets() {
        int nSets = getNumberOfFeatureSets();
        if (nSets == 0) {
            return null;
        }
        if (combineFeaturesIntoOneFile) {
            return new String[] { "comb" };
        }

        String[] ext = new String[nSets];
        int n = 0;
        // *VERY IMPORTANT*: the order here **must** be identical to the order
        // of the features returned by extractAudioFile
        if (extractSSD) {
            ext[n++] = "ssd";
        }
        if (extractRP) {
            ext[n++] = "rp";
        }
        if (extractRH) {
            ext[n++] = "rh";
        }

        return ext;
    }

    public int getNumberOfOutputFormats() {
        int f = 0;
        if (writeSOMLib) {
            f++;
        }
        if (writeXML) {
            f++;
        }
        if (writeOSIRIS) {
            f++;
        }
        if (writeWekaARFF) {
            f++;
        }
        return f;
    }

    public int determineNumberOfOutputFiles() {
        return getNumberOfFeatureSets() * getNumberOfOutputFormats();
    }

}
