/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.distance;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import at.tuwien.ifs.feature.extraction.audio.data.FeatureVectorData;
import at.tuwien.ifs.feature.extraction.audio.input.SOMLibFormatReader;
import at.tuwien.ifs.feature.extraction.audio.util.MyLogger;

/**
 * Copyright Vienna University of Technology
 * 
 * @author Thomas Lidy
 * @version $Id: DistanceMatrixComputer.java 173 2010-06-17 16:28:05Z mayer $ inspired by sources by
 *          Rudolf Mayer
 */
public class DistanceMatrixComputer {

    /**
     * @param args
     */
    public static void main(String[] args) {

        // String inFilename = args[0];

        // String inFilename =
        // "D:\\ProjekteIFS\\multicollection-evaluation\\bestof\\GTZAN.ssd";
        // String inFilename =
        // "/home/lidy/Experiments/multicollection-evaluation/bestof/GTZAN.ssd";
        String inFilename = "test-dist/GTZAN.ssd";

        /*
         * String collection = args[1]; String[] featureMethods; String outputDir = "results"; if
         * (args.length > 2) { outputDir = args[2]; } if (!outputDir.endsWith(File.separator)) {
         * outputDir += File.separator; }
         */

        // Test parameters
        DistanceMetric[] metrics = new DistanceMetric[] { new L1Metric(), new L2Metric() };

        SOMLibFormatReader somReader = null;
        try {
            somReader = new SOMLibFormatReader(new File(inFilename));
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        FeatureVectorData[] vectors = somReader.getFeatureVectors();

        DistanceMatrixComputer dmc;
        try {
            dmc = new DistanceMatrixComputer(vectors, metrics);

            /* do an example query with an index from the input vectors */
            int queryIndex = 0;
            int N = 10;
            MyLogger.info("Query: " + queryIndex + ": " + vectors[queryIndex].getFileLabel());

            List<VectorDistance> distancesList = dmc.getNearestNwithDistances(queryIndex, null, N);

            // FeatureVectorData[] result = new FeatureVectorData[N];

            for (int i = 0; i < distancesList.size(); i++) {
                MyLogger.info(i + " [" + distancesList.get(i).getDistance() + "]: "
                        + distancesList.get(i).getFeatureVector().getFileLabel());
            }

        } catch (MetricException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        // SimpleDateFormat simpleDateFormat = new
        // SimpleDateFormat("dd/MM, HH:mm:ss");

        /*
         * Date startDate = new Date(); PrintWriter writer = new PrintWriter(new
         * FileWriter(outputDir + collection + "-" + featureMethods[featureIndex] + "_results_" +
         * new SimpleDateFormat("yyyyMMdd_HHmmss").format(startDate) + ".csv"));
         * writer.println("Collection: " + collection); writer.println("Feature: " +
         * featureMethods[featureIndex]); writer.println("Started: " + startDate); String
         * baseFileName = root + File.separator + collection + File.separator; baseFileName = root +
         * File.separator + collection + File.separator; SOMLibClassInformation classInfo = new
         * SOMLibClassInformation(baseFileName + collection + ".clsinfo"); String vectorBaseFileName
         * = baseFileName + "vec" + File.separator + collection + "."; // DistanceMetric[] metrics =
         * new DistanceMetric[]; writer.print("\n\nFeatures,Metric,Alpha,"); for (int k = 0; k <
         * precisions.length; k++) { writer.print("Precision at " + precisions[k]); if (k + 1 <
         * precisions.length) { writer.print(","); } } writer.println("");
         */

    }

    private double[][] distanceMatrix = null;

    private FeatureVectorData[] vectors = null;

    public DistanceMatrixComputer(FeatureVectorData[] vectors, DistanceMetric metric)
            throws MetricException {
        this(vectors, new DistanceMetric[] { metric });
    }

    public DistanceMatrixComputer(FeatureVectorData[] vectors, DistanceMetric[] metrics)
            throws MetricException {
        this.vectors = vectors;
        int numVectors = vectors.length;

        /* compute distance matrix (matrices) */

        distanceMatrix = new double[numVectors][numVectors];

        for (int i = 0; i < numVectors; i++) {
            // System.out.print(i + " - ");
            for (int j = i + 1; j < numVectors; j++) { // starting with i+1
                // computes only upper
                // half of matrix;
                // matrix will be symmetric anyway

                /* TODO calculate different metrics */
                for (int metricIndex = 0; metricIndex < metrics.length; metricIndex++) {
                    distanceMatrix[i][j] = metrics[metricIndex].distance(vectors[i].getVector(),
                            vectors[j].getVector());
                    distanceMatrix[j][i] = distanceMatrix[i][j];
                }

            }
            // set diagonal to 0
            distanceMatrix[i][i] = 0;
        }
    }

    /*
     * writer.print(featureMethods[featureIndex] + "," + metrics[metricIndex] + "," + alpha + ",");
     * for (int i = 0; i < averagePrecisions.length; i++) { // caluclate average precision for each
     * feature averagePrecisions[i] = averagePrecisions[i] / data.numVectors();
     * writer.print(averagePrecisions[i]); if (i + 1 < averagePrecisions.length) {
     * writer.print(","); } } writer.println(""); writer.flush(); } writer.println("\n"); }
     * writer.println("\n\n"); Date endDate = new Date(); double duration = endDate.getTime() -
     * startDate.getTime() / 1000; String endMessage = "Finished: " + endDate + " (" + duration /
     * (60 * 60) + " minutes)"; System.out.println(endMessage); writer.println(endMessage);
     * writer.close(); }
     */

    // TODO optimize the code by not sorting the entire row, but using if
    // statements
    // for directly determining the Top N (see Code in somtoolbox)
    public List<VectorDistance> getNearestNwithDistances(int inputIndex, DistanceMetric metric,
            int N) throws MetricException {
        // InputDatum input = getInputDatum(inputIndex);
        ArrayList<VectorDistance> distances = new ArrayList<VectorDistance>();

        if (distanceMatrix != null) // take results from distanceMatrix
        {
            for (int i = 0; i < distanceMatrix[inputIndex].length; i++) {
                if (inputIndex != i) {
                    distances.add(new VectorDistance(distanceMatrix[inputIndex][i], vectors[i]));
                }
            }
        } else // compute distances on the fly
        {
            for (int i = 0; i < vectors.length; i++) {
                if (inputIndex != i) {
                    distances.add(new VectorDistance(metric.distance(
                            vectors[inputIndex].getVector(), vectors[i].getVector()), vectors[i]));
                }
            }
        }
        Collections.sort(distances);
        // TODO throw exception if distances.size < N
        return distances.subList(0, N);
    }

    public FeatureVectorData[] getNearestN(int inputIndex, DistanceMetric metric, int N)
            throws MetricException {
        List<VectorDistance> distancesList = getNearestNwithDistances(inputIndex, metric, N);

        FeatureVectorData[] result = new FeatureVectorData[N];

        for (int i = 0; i < distancesList.size(); i++) {
            result[i] = distancesList.get(i).getFeatureVector();
        }

        return result;
    }

}
