/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.input;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

/**
 * Copyright Vienna University of Technology<br>
 * <br>
 * reads an ASCII file in csv format (comma separated values).<br>
 * separator can be any character (not only comma), character has to be provided to readLine method
 * 
 * @author Thomas Lidy
 * @version $Id: CSVInputReader.java 172 2010-06-17 16:21:16Z mayer $
 */
public class CSVInputReader {

    BufferedReader reader;

    public CSVInputReader(File file) throws IOException {
        reader = new BufferedReader(new FileReader(file));
    }

    /**
     * reads a line
     * 
     * @param separator separator to split line into elements
     * @return an array of strings containing field elements separated by given separator, null if
     *         file is at the end
     * @throws IOException
     */
    public String[] readLine(String separator) throws IOException {
        String line = reader.readLine();
        if (line == null)
            return null;
        return line.split(separator);
    }

    public void close() throws IOException {
        reader.close();
    }
}
