/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.input;

import java.io.File;
import java.io.IOException;

import at.tuwien.ifs.feature.extraction.audio.data.FeatureVectorData;

/**
 * Copyright Vienna University of Technology<br>
 * <br>
 * reads in ID/URL List from DELOS DLMS OSIRIS (Basel) system.<br>
 * format is e.g.: 1@audio ; http://delos:dlms@isisdemo.cs.unibas.ch/ISIS.AUDIO/Music/A3/Exile on
 * Coldharbour Lane/A3 - Ain't Goin' to Goa.mp3
 * 
 * @author Thomas Lidy
 * @version $Id: OSIRISFileListInputReader.java 172 2010-06-17 16:21:16Z mayer $
 */
public class OSIRISFileListInputReader extends AbstractFileListReader {

    public OSIRISFileListInputReader(File file) throws IOException {
        readFile(file);
    }

    private void readFile(File file) throws IOException {
        CSVInputReader cvsreader = new CSVInputReader(file);
        // Vector javaVector = new Vector();

        FeatureVectorData featureVector;

        String[] lineElements;

        /* PROCESS HEADER with arbitrary number of lines starting with $ */
        while ((lineElements = cvsreader.readLine(";")) != null) {
            if (lineElements.length != 2) {
                throw new IOException(
                        "Unexpected file format: 2 elements per line separated by ; expected.");
            }
            featureVector = new FeatureVectorData();
            featureVector.setId(lineElements[0].trim());
            featureVector.setFileLabel(lineElements[1].trim());

            metaDataList.add(featureVector);
        }

        cvsreader.close();
    }

}
