/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.input;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import at.tuwien.ifs.feature.extraction.audio.data.FeatureVectorData;

/**
 * Copyright Vienna University of Technology
 * 
 * @author Thomas Lidy
 * @version $Id: SOMLibFormatReader.java 173 2010-06-17 16:28:05Z mayer $ inspired by somtoolbox
 */
public class SOMLibFormatReader {

    private FeatureVectorData[] vectors;

    private int numVectors = 0;

    private int vecDim = 0;

    public SOMLibFormatReader(File file) throws IOException {
        readVectorFile(file);
    }

    private void readVectorFile(File file) throws IOException {
        BufferedReader br = new BufferedReader(new FileReader(file));

        String line;
        String header, content;
        // int ydim = 1;

        /* PROCESS HEADER with arbitrary number of lines starting with $ */
        while ((line = br.readLine()) != null) {
            if (!(line.startsWith("$")))
                break;

            String[] lineElements = line.split("[ \t]");
            if (lineElements.length < 2) {
                // Logger.getLogger("at.ec3.somtoolbox").severe("Header in input vector file corrupt!");
                throw new IOException("Header in input vector file corrupt!");
            }
            header = lineElements[0];
            content = lineElements[1];
            /*
             * if (header.equals("$TYPE")) {
             * // do nothing
             * } else if (header.equals("$DATA_TYPE")) {
             * // determine type of vector data, if data is audio set sparsity
             * to false
             * String[] subtypes = content.split("-", 2);
             * String content_type = subtypes[0];
             * String content_subtype = subtypes[1];
             * if (content_type.equals("audio")) {
             * }
             * } else
             */
            if (header.equals("$XDIM")) {
                numVectors = Integer.parseInt(content);
            } /*
              * else if (header.equals("$YDIM")) {
              * ydim = Integer.parseInt(content);
              * }
              */
            else if (header.equals("$VEC_DIM")) {
                vecDim = Integer.parseInt(content);
            }
        }

        if ((vecDim == 0) || (numVectors == 0))
            throw new IOException(
                    "Header in input vector file corrupt! Could not find either $XDIM or $VEC_DIM line.");
        // numVectors *= ydim;

        /* PROCESS REMAINDER OF FILE */

        vectors = new FeatureVectorData[numVectors];
        double[] vector = new double[vecDim];

        int index = 0;

        while ((line != null) && (index < numVectors)) {
            // progressWriter.progress(index);
            String[] lineElements = line.split("[ \t]", vecDim + 1); // split
            // max.
            // vecDim
            // + 1
            // times
            // to
            // avoid
            // splitting
            // of
            // filenames
            // with
            // spaces
            if (lineElements.length != (vecDim + 1)) {
                String errorMessage = "Input vector file format corrupt in vector number " + index
                        + ". Aborting.";
                throw new IOException(errorMessage);
            } else { // vector syntax ok. checking number format and calculating
                // meanVector.
                try {

                    for (int ve = 0; ve < vecDim; ve++) {
                        vector[ve] = Double.parseDouble(lineElements[ve]);
                    }
                    String label = lineElements[vecDim];

                    vectors[index] = new FeatureVectorData();
                    vectors[index].setFileLabel(label);
                    vectors[index].setVector(vector);
                    index++;

                } catch (NumberFormatException e) {
                    String errorMessage = "Input vector number format corrupt in vector number "
                            + index + ".";
                    throw new IOException(errorMessage);
                }
            }
            line = br.readLine();
        }

        br.close();

        if (index != numVectors) {
            String errorMessage = "Input vector file format corrupt. Incorrect number of vectors in file.";
            throw new IOException(errorMessage);
        }
    }

    public FeatureVectorData[] getFeatureVectors() {
        return vectors;
    }
}
