/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.output;

import java.io.IOException;

import at.tuwien.ifs.feature.extraction.audio.data.FeatureVectorData;

/**
 * Copyright Vienna University of Technology<br>
 * <br>
 * Wwrites feature vectors in DELOS DLMS OSIRIS (Basel) format, i.e. per line: <br>
 * &lt;id&gt;,RP,space separated float values
 * 
 * @author Thomas Lidy
 * @version $Id: OSIRISFormatWriter.java 172 2010-06-17 16:21:16Z mayer $
 */
public class OSIRISFormatWriter extends AbstractDataWriter {

    public OSIRISFormatWriter(String outFileName) throws IOException {
        super(outFileName);
    }

    @Override
    public void writeVector(FeatureVectorData featureVector) throws IOException {
        double[][] matrix = featureVector.getMatrixData();
        int nRows = matrix.length;
        int nCols = matrix[0].length;
        int vectorDim = nRows * nCols;

        checkVectorDim(vectorDim);

        try {
            outFile.write(featureVector.getId() + ",");
            outFile.write(dataTypeID + ",");

            /* write matrix column-wise, as Matlab does when vectorizing */

            for (int c = 0; c < nCols; c++) {
                for (int r = 0; r < nRows; r++) {
                    outFile.write(formatDouble.format(matrix[r][c]) + " ");
                }
            }

            outFile.newLine();

        } catch (IOException e) {
            throw new IOException("Problem writing OSIRIS vector file " + fileName + "! "
                    + e.getMessage());
        }
    }

}
