/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.output;

import java.io.IOException;

import at.tuwien.ifs.feature.extraction.audio.data.FeatureVectorData;

/**
 * Copyright Vienna University of Technology
 * 
 * @author Thomas Lidy
 * @version $Id: XMLFormatWriter.java 173 2010-06-17 16:28:05Z mayer $
 */
public class XMLFormatWriter extends AbstractDataWriter {

    public XMLFormatWriter(String outFileName) throws IOException {
        super(outFileName);
        writeHeader();
    }

    private void writeHeader() throws IOException {
        outFile.write("<!-- AudioDescriptor XML Format -->" + nl);
        outFile.write("<!-- (c) Vienna University of Technology, Institute of Software Technology and Interactive Systems, MIR group -->"
                + nl);
        outFile.write("<!-- file written by at.tuwien.ifs.feature.extraction.audio -->" + nl + nl);

        outFile.write("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>" + nl + nl);

        outFile.write("<AudioDescription>" + nl);
    }

    @Override
    public void setBaseDirectory(String baseDir) {
        super.setBaseDirectory(baseDir);
        try {
            outFile.write("\t<DataLocation>" + nl);
            outFile.write("\t\t" + baseDir + nl);
            outFile.write("\t</DataLocation>" + nl);
        } catch (IOException e) {
            System.err.println("Could not write base directory to XML file!");
        }
    }

    private void writeFooter() throws IOException {
        outFile.write("</AudioDescription>" + nl);
    }

    @Override
    public void close() throws IOException {
        writeFooter();
        super.close();
    }

    // public void writeMatrix(double[][] matrix, String label) throws
    // IOException
    // {
    @Override
    public void writeVector(FeatureVectorData featureVector) throws IOException {
        double[][] matrix = featureVector.getMatrixData();
        int nRows = matrix.length;
        int nCols = matrix[0].length;
        int vectorDim = nRows * nCols;

        checkVectorDim(vectorDim);

        try {
            outFile.write("<AudioFile>" + nl);
            outFile.write("\t<Filename>" + nl);
            outFile.write("\t\t" + featureVector.getFileLabel() + nl); // FIXME:
            // this
            // may
            // introduce
            // problems
            // with
            // Filenames
            // containing
            // spaces
            // (?)
            outFile.write("\t</Filename>" + nl);

            outFile.write("\t<Descriptor ");
            if (dataTypeID != null)
                outFile.write("type=\"" + dataTypeID + "\" ");
            outFile.write("vecdim=\"" + vectorDim + "\" dim=\"" + nRows + " " + nCols + "\">");

            /* write matrix column-wise, as Matlab does when vectorizing */// FIXME:
            // really?

            for (int c = 0; c < nCols; c++) {
                outFile.write(nl + "\t\t");
                for (int r = 0; r < nRows; r++) {

                    outFile.write(formatFloat.format(matrix[r][c]) + " ");
                }
            }

            outFile.write(nl + "\t</Descriptor>" + nl);
            outFile.write("</AudioFile>" + nl);

        } catch (IOException e) {
            throw new IOException("Problem writing XML file " + fileName + "! " + e.getMessage());
        }
    }

}
