/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.spectrum;

/**
 * Copyright Vienna University of Technology
 * 
 * @author Simon Diesenreiter for M2K, altered by Thomas Lidy
 * @version $Id: BarkScale.java 173 2010-06-17 16:28:05Z mayer $
 */

public class BarkScale {

    private static final double[] BARK_LIMITS = { 100, 200, 300, 400, 510, 630, 770, 920, 1080,
            1270, 1480, 1720, 2000, 2320, 2700, 3150, 3700, 4400, 5300, 6400, 7700, 9500, 12000,
            15500, 22050 };

    int sampleRate;

    double binSizeHz;

    public BarkScale(int sampleRate, int fft_window_size) {
        this.sampleRate = sampleRate;
        this.binSizeHz = (double) sampleRate / (double) fft_window_size;
    }

    public BarkScale(int sampleRate, double binSize_inHz) {
        this.sampleRate = sampleRate;
        this.binSizeHz = binSize_inHz;
    }

    public int getNumberOfBarkBands() {
        return getNumberOfBarkBands(this.sampleRate);
    }

    public static int getNumberOfBarkBands(int sampleRate) {
        /* determine number of used bark bands */
        int i = 0;
        while ((i < BARK_LIMITS.length) && (BARK_LIMITS[i] <= sampleRate / 2)) {
            i++;
        }
        return i + 1;
    }

    public String getModuleInfo() {
        return "<p>Overview: dieses modul definiert ein array fuer die Bark-Skala "
                + "</p><p>und addiert die werte aus dem input die hineinfallen zusammen"
                + "</p><p>Detailed Description: "
                + "</p><p>Data Handling: the input data IS modified.";
    }

    // gibt im info-feld den "common Name" an
    public String getModuleName() {
        return "BarkAll";
    }

    // gibt im "info"-feld eine beschreibung der eingaenge
    public String getInputInfo(int i) {
        switch (i) {
            case 0:
                return "Double 2DArray ";
                // case 1:
                // return "java.lang.Integer";
            default:
                return "No such input";
        }
    }

    // hier werden die eingaenge definiert (wieviele u welcher typ)
    public String[] getInputTypes() {
        String[] types = {
        // "[[D", "java.lang.Integer"};
        "[[D" };
        return types;
    }

    // gibt im info-feld den namen des inputs an UND benennt die eingaenge des
    // modules selbst
    public String getInputName(int i) {
        switch (i) {
            case 0:
                return "Double 2DArray";
                // case 1:
                // return "java.lang.Integer";
            default:
                return "NO SUCH INPUT!";
        }
    }

    // gibt im output-feld den namen des outputs an UND benennt die ausgaenge
    // des modules selbst
    public String getOutputName(int i) {
        switch (i) {
            case 0:
                return "Double 2DArray";
            default:
                return "NO SUCH OUTPUT!";
        }

    }

    // gibt im "info"-feld eine beschreibung der ausgaenge
    public String getOutputInfo(int i) {
        switch (i) {
            case 0:
                return "Double 2DArray ";
            default:
                return "No such input";
        }
    }

    // hier werden die ausgaenge definiert (wieviele u welcher typ)
    public String[] getOutputTypes() {
        String[] types = { "[[D" };
        return types;
    }

    // bei samplerate von 11025 hat man max frequenz von 5512,5hz (vgl
    // samplingtheorem)
    // =>bei framesize von 256 entspricht jeder wert d x-achse einem spektrum
    // von ~ 43.07hz
    // double[][] out = null;
    // int SampleRate;
    // int FrameSize;
    // double binSize = (11025 / 256); //binsize =~ 43hz for ALL SampleRates

    /**
   * 
   */

    public double[] apply(double[] spectrumData) {

        if (spectrumData == null) {
            return null;
        }

        int specLen = spectrumData.length;
        int barkLen = getNumberOfBarkBands();

        // System.out.println("---Bark FrameSize: " + specLen);
        // System.out.println("---# bark bands: " + barkLen);

        // initialize output vector
        double[] barkSpec = new double[barkLen];

        // f ... frequency band, curFreq ... frequency represented by band f
        // f starts from 1 skipping DC component
        int b = 0; // b ... bark band index
        barkSpec[b] = 0;
        for (int f = 1; f < specLen / 2 + 1; f++) {
            double curFreq = f * binSizeHz;
            // System.out.println("curFreq: " + curFreq + " bark: " + b);
            while (curFreq > BARK_LIMITS[b]) {
                b++;
                barkSpec[b] = 0;
            }

            barkSpec[b] += spectrumData[f];
        }

        return barkSpec;
    }
}
