/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.util;

/**
 * Interface for Loggers used by MyLogger.
 * 
 * @author Jakob Frank
 */
public interface LogHandler {

    enum LogGrade {
        /**
         * Most chatty
         */
        EXTRA_INFO {
            @Override
            public String toString() {
                return "";
            }
        },
        /**
         * Verboose
         */
        INFO {
            @Override
            public String toString() {
                return "";
            }
        },
        /**
         * Warnings
         */
        WARNING {
            @Override
            public String toString() {
                return super.toString() + ": ";
            }
        },
        /**
         * Errors
         */
        ERROR {
            @Override
            public String toString() {
                return super.toString() + ": ";
            }
        },
        /**
         * Serious errors.
         */
        SEVERE {
            @Override
            public String toString() {
                return super.toString() + ": ";
            }
        },
        /**
         * Use the same grade as in the last call, and also stay in the same line! (Only possible if
         * the logger has not been flushed since the last log)
         */
        SAME {
            @Override
            public String toString() {
                return "";
            }
        }
    };

    /**
     * Append a log-line.
     * 
     * @param message The message to append.
     * @param grade The grade.
     * @see LogGrade
     */
    void appendLog(String message, LogGrade grade);

    /**
     * Flush the log. Do really print it.
     */
    void flushLog();

    /**
     * Set the Log-Grade. Only messages that are more or equal important than the given grade will
     * be shown.
     * 
     * @param grade The new log grade.
     */
    void setLogGrade(LogGrade grade);

    /**
     * Flush all log channels.
     * 
     * @see #flushLog()
     */
    void flushAllLogs();
}
