/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.util;

import java.io.PrintStream;
import java.util.Hashtable;

/**
 * Loggs to a PrintStream, System.out per default.
 * 
 * @author frank
 */
public class PrintStreamLogger implements LogHandler {

    private LogGrade grade;

    private final Hashtable<String, StringBuilder> channels;

    private final PrintStream sink;

    private String lineSeperator = "\n";

    public PrintStreamLogger() {
        this(System.out);
    }

    public PrintStreamLogger(PrintStream ps) {
        grade = LogGrade.EXTRA_INFO;
        channels = new Hashtable<String, StringBuilder>();
        sink = ps;
    }

    private synchronized StringBuilder getStringBuilder() {
        Thread t = Thread.currentThread();
        String key = t.getName() + "-" + t.getId();
        synchronized (channels) {
            StringBuilder sb = channels.get(key);
            if (sb == null) {
                sb = new StringBuilder();
                channels.put(key, sb);
            }
            return sb;
        }
    }

    /*
     * (non-Javadoc)
     * @see at.tuwien.ifs.feature.extraction.audio.util.LogHandler#appendLog(java.lang .String,
     * at.tuwien.ifs.feature.extraction.audio.util.LogHandler.LogGrade)
     */
    @Override
    public void appendLog(String message, LogGrade grade) {
        if (grade.compareTo(this.grade) < 0)
            return;

        StringBuilder sb = getStringBuilder();
        synchronized (sb) {
            if (grade == LogGrade.SAME) {
                int pos = sb.lastIndexOf(lineSeperator);
                if (pos > 0) {
                    sb.replace(pos, sb.capacity(), message).append(lineSeperator);
                } else {
                    sb.append(message).append(lineSeperator);
                }
            } else {
                sb.append(grade).append(message).append(lineSeperator);
            }
        }
    }

    /*
     * (non-Javadoc)
     * @see at.tuwien.ifs.feature.extraction.audio.util.LogHandler#flushLog()
     */
    @Override
    public void flushLog() {
        StringBuilder sb = getStringBuilder();
        synchronized (sb) {
            sink.print(sb);
            sink.flush();
            sb.setLength(0);
        }
    }

    @Override
    public void setLogGrade(LogGrade grade) {
        this.grade = grade;
    }

    @Override
    public void flushAllLogs() {
        synchronized (channels) {
            for (StringBuilder sb : channels.values()) {
                synchronized (sb) {
                    sink.print(sb);
                    sink.flush();
                    sb.setLength(0);
                }
            }
        }
    }

}
