/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.vis;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.filechooser.FileFilter;

/**
 * Copyright Vienna University of Technology
 * 
 * @author Thomas Lidy
 * @version $Id: InputOutputSettingsPanel.java 172 2010-06-17 16:21:16Z mayer $
 */
public class InputOutputSettingsPanel extends javax.swing.JPanel implements ActionListener {

    private static final long serialVersionUID = 1L;

    private static final int BORDER = 20;

    private JLabel lblInput;

    private JButton btnOpenDir;

    private JSeparator jSeparator1;

    private JButton btnOpenFile;

    private JTextField txtInput;

    private JPanel panelInput;

    private JPanel panelInput2;

    private JLabel lblExtensions;

    private JTextField txtExtensions;

    private JPanel panelOutput;

    private JLabel lblOutput;

    private JTextField txtOutputDir;

    private JButton btnOpenOutDir;

    private JTextField txtProjectName;

    private JLabel lblProject;

    private final String OPEN_INFILE = "openInFile";

    private final String OPEN_INDIR = "openInDir";

    private final String OPEN_OUTDIR = "openOutDir";

    /**
     * Auto-generated main method to display this JPanel inside a new JFrame.
     */
    public static void main(String[] args) {
        JFrame frame = new JFrame();
        frame.getContentPane().add(new InputOutputSettingsPanel());
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }

    public InputOutputSettingsPanel() {
        super();
        initGUI();
    }

    private void initGUI() {

        setBorder(BorderFactory.createEmptyBorder(BORDER, BORDER, BORDER, BORDER));

        BoxLayout thisLayout = new BoxLayout(this, javax.swing.BoxLayout.Y_AXIS);
        this.setLayout(thisLayout);

        // setPreferredSize(new Dimension(400, 300));

        lblInput = new JLabel();
        this.add(lblInput);
        lblInput.setText("Input file or directory:");

        panelInput = new JPanel();
        BoxLayout panelInputLayout = new BoxLayout(panelInput, javax.swing.BoxLayout.X_AXIS);
        this.add(panelInput);
        panelInput.setLayout(panelInputLayout);

        txtInput = new JTextField();
        panelInput.add(txtInput);

        btnOpenFile = new JButton();
        panelInput.add(btnOpenFile);
        btnOpenFile.setText("File ...");
        btnOpenFile.setActionCommand(OPEN_INFILE);
        btnOpenFile.addActionListener(this);

        btnOpenDir = new JButton();
        panelInput.add(btnOpenDir);
        btnOpenDir.setText("Dir ...");
        btnOpenDir.setActionCommand(OPEN_INDIR);
        btnOpenDir.addActionListener(this);

        jSeparator1 = new JSeparator();
        panelInput.add(jSeparator1);

        panelInput2 = new JPanel();
        BoxLayout panelInputLayout2 = new BoxLayout(panelInput2, javax.swing.BoxLayout.X_AXIS);
        this.add(panelInput2);
        panelInput2.setLayout(panelInputLayout2);

        lblExtensions = new JLabel();
        panelInput2.add(lblExtensions);
        lblExtensions.setText("Extensions: ");

        txtExtensions = new JTextField();
        panelInput2.add(txtExtensions);
        txtExtensions.setText("wav,au,mp3");

        lblOutput = new JLabel();
        this.add(lblOutput);
        lblOutput.setText("Output directory:");

        panelOutput = new JPanel();
        BoxLayout panelOutputLayout = new BoxLayout(panelOutput, javax.swing.BoxLayout.X_AXIS);
        this.add(panelOutput);
        panelOutput.setLayout(panelOutputLayout);

        txtOutputDir = new JTextField();
        panelOutput.add(txtOutputDir);
        // txtOutputDir.setText("");

        btnOpenOutDir = new JButton();
        panelOutput.add(btnOpenOutDir);
        btnOpenOutDir.setLayout(null);
        btnOpenOutDir.setText("Dir ...");
        btnOpenOutDir.setActionCommand(OPEN_OUTDIR);
        btnOpenOutDir.addActionListener(this);

        lblProject = new JLabel();
        this.add(lblProject);
        lblProject.setText("Base name of output files: ");

        txtProjectName = new JTextField();
        this.add(txtProjectName);
        txtProjectName.setText("project");
    }

    private String selectFile() {
        JFileChooser fc = new JFileChooser();

        fc.addChoosableFileFilter(new FileFilter() {

            @Override
            public boolean accept(File f) {
                if (f.isDirectory())
                    return true;
                String lcName = f.getName().toLowerCase();
                if (lcName.endsWith(".wav") || lcName.endsWith(".mp3") || lcName.endsWith(".au"))
                    return true;
                return false;
            }

            @Override
            public String getDescription() {
                return "Audio Files (.wav, .au, .mp3)";
            }
        });

        if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFile().toString();
        } else
            return null;
    }

    private String selectDirectory() {
        JFileChooser fc = new JFileChooser();

        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFile().toString();
        } else
            return null;
    }

    public String getInputDirectory() {
        return txtInput.getText();
    }

    public String getInputExtensions() {
        return txtExtensions.getText();
    }

    public String getOutputDirectory() {
        return txtOutputDir.getText();
    }

    public String getOutputBasename() {
        return txtProjectName.getText();
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand() == OPEN_INFILE) {
            String dir = selectFile();
            if (dir != null)
                txtInput.setText(dir);
        }
        if (e.getActionCommand() == OPEN_INDIR) {
            String dir = selectDirectory();
            if (dir != null)
                txtInput.setText(dir);
        }
        if (e.getActionCommand() == OPEN_OUTDIR) {
            String dir = selectDirectory();
            if (dir != null)
                txtOutputDir.setText(dir);
        }
    }

}
