/*
 * Copyright 2004-2010 Information & Software Engineering Group (188/1)
 *                     Institute of Software Technology and Interactive Systems
 *                     Vienna University of Technology, Austria
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.ifs.tuwien.ac.at/mir/index.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package at.tuwien.ifs.feature.extraction.audio.vis;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.text.NumberFormat;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

/**
 * Copyright Vienna University of Technology<br>
 * <br>
 * * Matrix value visualization derived from RhythmPatternsVisWindow class in somtoolbox now
 * resizable <br>
 * TODO: constructor with resolution info: show better status text
 * 
 * @author Thomas Lidy
 * @version $Id: MatrixVisWindow.java 172 2010-06-17 16:21:16Z mayer $
 */
public class MatrixVisWindow extends JDialog implements MouseListener, MouseMotionListener {

    private static final long serialVersionUID = 1L;

    private int xdim = 0; // matrix dimensions

    private int ydim = 0;

    private double[][] matrix; // the matrix

    private double minv = 0; // min. value of matrix

    private double maxv = 0; // max. value of matrix

    private double diff = 0; // difference between min. and max. value of matrix

    // FIXME: bx, by obsolete now
    private static final int DEFAULT_bx = 1; // default graphics block size in

    // pixel (x)
    private static final int DEFAULT_by = 1; // default graphics block size in

    // pixel (y)

    private int bx = DEFAULT_bx; // graphics block size in pixel (x)

    private int by = DEFAULT_by; // graphics block size in pixel (y)

    private boolean display_status = true; // show info in status bar

    private DrawingPane drawingPane;

    private Dimension drawingPaneSize;

    private Color[] palette;

    BufferedImage bimage;

    private JLabel lblDescription;

    private JLabel status;

    private NumberFormat numform;

    /**
     * This constructor uses pre-defined hard-coded values; it is not recommended to use this
     * constructor it is intended for backward compatibility for RP feature vector files without the
     * $DATA_TYPE header
     */
    public MatrixVisWindow(Frame parent, double[][] matrix) {
        this(parent, matrix, " ");
    }

    /**
     * @param matrix a matrix given as double[][] array
     * @param description
     */
    public MatrixVisWindow(Frame parent, double[][] matrix, String description) {
        // this(matrix, description, DEFAULT_bx, DEFAULT_by);
        // }

        /**
         * @param matrix a matrix given as double[][] array
         * @param blocksize_x the blocksize in pixels for a matrix data point (x)
         * @param blocksize_y the blocksize in pixels for a matrix data point (y)
         */
        // public MatrixVisWindow(double[][] matrix, String description, int
        // blocksize_x, int blocksize_y)
        // {
        super(parent);

        this.matrix = matrix;
        this.xdim = matrix.length;
        this.ydim = matrix[0].length;
        // this.bx = blocksize_x;
        // this.by = blocksize_y;

        calcMinMax();
        initPaint();
        initGUI(description);

        setVisible(true);
    }

    private void initGUI(String description) {
        // Handle window closing correctly.
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        // setResizable(false);
        setTitle(description); // "MatrixVis");

        setLayout(new BorderLayout());

        lblDescription = new JLabel(description + " (" + xdim + "x" + ydim + ")");
        getContentPane().add(lblDescription, BorderLayout.NORTH);

        status = new JLabel("status");
        getContentPane().add(status, BorderLayout.SOUTH);

        // Set up the drawing area.
        drawingPane = new DrawingPane();
        drawingPane.setBackground(Color.white);
        // drawingPane.setPreferredSize(getMatrixVisSize());

        int default_xsize = 300;
        int default_ysize = (default_xsize * ydim) / xdim;
        while (default_ysize > 1000)
            default_ysize /= 2;
        drawingPane.setPreferredSize(new Dimension(default_xsize, default_ysize));
        drawingPane.addMouseListener(this);
        drawingPane.addMouseMotionListener(this);
        getContentPane().add(drawingPane, BorderLayout.CENTER);
        pack();
        drawingPaneSize = drawingPane.getSize();

        // initialize display number formats
        // numform_Hz = NumberFormat.getInstance();
        // numform_Hz.setMaximumFractionDigits(2);
        numform = NumberFormat.getInstance();
        numform.setMaximumFractionDigits(6);
    }

    private class DrawingPane extends JPanel {

        private static final long serialVersionUID = 1L;

        public DrawingPane() {
        }

        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            // paintMatrix(g);
            paintScaledImage(g);
        }
    }

    /**
     * @return size of matrix in pixels
     */
    public Dimension getMatrixVisSize() {
        // System.out.println("getmatrixout" + new Dimension(xdim*bx,
        // ydim*by).toString());
        return drawingPane.getSize(drawingPaneSize);
        // return new Dimension(xdim*bx, ydim*by);
    }

    private void initPaint() {
        palette = initMatlabPalette();
        bimage = getImage();
    }

    private void calcMinMax() {
        minv = Double.POSITIVE_INFINITY;
        maxv = Double.NEGATIVE_INFINITY;

        for (int x = 0; x < xdim; x++) {
            for (int y = 0; y < ydim; y++) {
                if (matrix[x][y] < minv)
                    minv = matrix[x][y]; // determine minimum value
                if (matrix[x][y] > maxv)
                    maxv = matrix[x][y]; // determine maximum value
            }
        }

        diff = maxv - minv;
    }

    /**
     * @return x dimension of RP matrix
     */
    public int getXdim() {
        return xdim;
    }

    /**
     * @return y dimension of RP matrix
     */
    public int getYdim() {
        return ydim;
    }

    /**
     * @return matrix as double[][] array
     */
    public double[][] getMatrix() {
        return matrix;
    }

    /**
     * @return MatrixVis as a BufferedImage
     */
    public BufferedImage getImage() {
        BufferedImage img = new BufferedImage(xdim * bx, ydim * by, BufferedImage.TYPE_INT_RGB);
        paintMatrix(img.createGraphics());
        return img;
    }

    public void paintScaledImage(Graphics g) {
        // getting the size is not needing in resizing events (g containing the
        // correct size to repaint)
        // but when another window covers the dialog partly, we need to repaint
        // the whole pane
        // This version of getSize() is useful to avoid allocating a new
        // Dimension object on the heap.
        drawingPaneSize = drawingPane.getSize(drawingPaneSize);
        g.drawImage(bimage, 0, 0, drawingPaneSize.width, drawingPaneSize.height, Color.WHITE, this);
    }

    /**
     * paints on Graphics object provided (either by local method or by a Dialog window)
     */
    // public void paint(Graphics g) {
    public void paintMatrix(Graphics g) {

        int xpos, ypos;
        int colind;
        int paldim = palette.length - 1;

        if (diff == 0.0)
            return; // would cause div/0 //TODO throw exception, or paint a
        // rectangle with color 0

        xpos = 0;
        ypos = (ydim - 1) * by;

        for (int j = 0; j < ydim; j++) {
            for (int i = 0; i < xdim; i++) {

                // System.out.print(mat[i][j] + " ");
                colind = (int) (((matrix[i][j] - minv) / diff) * (double) paldim);
                g.setColor(palette[colind]);
                g.fillRect(xpos, ypos, bx, by);
                xpos += bx;
            }
            // System.out.println();
            ypos -= by;
            xpos = 0;
        }
    }

    /**
     * handles mouse events: status display remains fixed after clicking (continues after 2nd
     * clicking)
     */
    public void mouseReleased(MouseEvent e) {
        display_status = !display_status;
        if (e.getClickCount() == 2) {
            int cx = (e.getX() * (xdim) / (drawingPaneSize.width));
            if ((cx >= xdim) || (cx < 0))
                return; // avoid out of bounds
            @SuppressWarnings("unused")
            LineVisWindow linevis = new LineVisWindow(matrix[cx], "pos: " + cx, minv, maxv);
        }
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
    }

    /**
     * handles mouse events: status display changes when mouse is moved over Matrix
     */
    public void mouseMoved(MouseEvent e) {
        if (!display_status)
            return;

        int cx = (e.getX() * (xdim) / (drawingPaneSize.width));
        int cy = ((drawingPaneSize.height - 1 - e.getY()) * (ydim) / (drawingPaneSize.height));
        if ((cx >= xdim) || (cy >= ydim) || (cx < 0) || (cy < 0))
            return; // avoid out of bounds

        // float mod_freq = (cx+1) * mod_freq_res;
        // let index start from 1
        // status.setText("band " + Integer.toString(cy + 1) + " / " +
        // numform_Hz.format(mod_freq) + " Hz: " +
        // numform_Amp.format(mat[cx][cy]));

        status.setText(cx + " / " + cy + ": " + numform.format(matrix[cx][cy]));
    }

    /**
     * this palette is from somtoolbox 0.4x (not used)
     */
    @SuppressWarnings("unused")
    private Color[] initPalette() {
        Color[] palette = new Color[128];

        for (int c = 0; c < palette.length / 2; c++) {
            int r = 0 + (int) (c * ((double) 255 / ((double) palette.length / 2)));
            int g = 255;
            int b = 0;
            palette[c] = new Color(r, g, b);
        }
        for (int c = palette.length / 2; c < palette.length; c++) {
            int r = 255 - (int) ((c - (palette.length / 2)) * ((double) 255 / ((double) palette.length / 2)));
            int g = 255 - (int) ((c - (palette.length / 2)) * ((double) 255 / ((double) palette.length / 2)));
            int b = 0 + (int) ((c - (palette.length / 2)) * ((double) 255 / ((double) palette.length / 2)));
            palette[c] = new Color(r, g, b);
        }

        return palette;
    }

    /**
     * this palette is the MATLAB jet colormap it consists of 6 fixed color values which are
     * interpolated through 64 values
     */
    private Color[] initMatlabPalette() {
        int i, r, g, b;
        Color[] palette = new Color[64];

        for (i = 0; i < 8; i++) {
            palette[i] = new Color(0, 0, 143 + i * 16);
        }
        g = 0;
        for (i = 8; i < 23; i++) {
            g = g + 16;
            palette[i] = new Color(0, g, 255);
        }
        r = 0;
        b = 255;
        for (i = 23; i < 39; i++) {

            palette[i] = new Color(r, 255, b);
            r = r + 16;
            b = b - 16;
        }
        g = 255;
        for (i = 39; i < 55; i++) {

            palette[i] = new Color(255, g, 0);
            g = g - 16;
        }
        r = 255;
        for (i = 55; i < 63; i++) {
            palette[i] = new Color(r, 0, 0);
            r = r - 16;
        }
        palette[63] = new Color(128, 0, 0);

        // for (i = 0; i < 64; i++)
        // {
        // System.out.println(Integer.toString(i) + ":\t" + palette[i].getRed()
        // + "\t" + palette[i].getGreen() + "\t" + palette[i].getBlue());
        // }

        return palette;
    }

}
