package at.tuwien.ifs.feature.extraction.audio;

import java.io.File;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import at.tuwien.ifs.commons.util.TestUtils;
import at.tuwien.ifs.feature.extraction.audio.data.FeatureExtractionOptions;
import at.tuwien.ifs.feature.extraction.audio.data.FeatureVectorData;
import at.tuwien.ifs.feature.extraction.audio.util.MyLogger;
import at.tuwien.ifs.feature.extraction.audio.util.PrintStreamLogger;

/**
 * @author Rudolf Mayer
 * @version $Id: TestAudioFeatureExtraction.java 205 2010-08-30 10:11:07Z frank $
 */
public class TestAudioFeatureExtraction {

    @Test
    public void testFeatureExtraction() throws Exception {

        // create a temp directory for writing the newly extracted files
        File tmpDir = new File(
                TestUtilsAudioFeatureExtraction.getResource("/").getParentFile().getParentFile().getAbsoluteFile()
                        + "/tmp");
        if (!tmpDir.exists()) {
            tmpDir.mkdirs();
        }
        MyLogger.registerLogHandler(new PrintStreamLogger());
        MyLogger.setAutoFlush(true);
        MyLogger.info("Writing to temporary directory " + tmpDir.getAbsolutePath());

        File samplesDirectory = TestUtilsAudioFeatureExtraction.getResource(TestUtilsAudioFeatureExtraction.RSC_AUDIO_SAMPLES);
        MyLogger.info("Reading sample files from " + samplesDirectory.getAbsolutePath());

        File baseDir = new File(TestAudioFeatureExtraction.class.getResource("/").toURI());

        String samplesDirRelative = samplesDirectory.getAbsolutePath();
        if (samplesDirRelative.startsWith(baseDir.getParentFile().getAbsolutePath())) {
            samplesDirRelative = samplesDirRelative.substring(baseDir.getParentFile().getAbsolutePath().length());
        }
        if (samplesDirRelative.startsWith(File.separator)) {
            samplesDirRelative = samplesDirRelative.substring(File.separator.length());
        }
        MyLogger.info("Relative sample dir: " + samplesDirRelative);

        // extract the data
        FeatureExtractionOptions opt = new FeatureExtractionOptions();
        String baseFileName = "/testExtraction";
        opt.outputBaseFilename = tmpDir.getAbsolutePath() + baseFileName;
        opt.inputBaseDir = samplesDirRelative;
        opt.graphicalEnvironment = false;
        opt.extractSSD = true;
        opt.writeSOMLib = true;
        opt.enableSOMLibGZip = false;

        MultipleFileExtractor extractor = new MultipleFileExtractor(1);
        List<FeatureVectorData> list = MultipleFileExtractor.createExtractionList(samplesDirectory,
                opt);
        Collections.sort(list, new Comparator<FeatureVectorData>() {
            @Override
            public int compare(FeatureVectorData o1, FeatureVectorData o2) {
                return o1.getFileLabel().compareTo(o2.getFileLabel());
            }
        });
        extractor.extractListOfFiles(list, opt, null);

        // now do the comparison
        final File dirCompare = TestUtilsAudioFeatureExtraction.getResource(TestUtilsAudioFeatureExtraction.RSC_AUDIO_SAMPLES_COMPARE);
        MyLogger.info("Loading comparison files from " + dirCompare.getAbsolutePath()
                + baseFileName);
        final String[] fileExtensionsOfFeatureSets = opt.getFileExtensionsOfFeatureSets();
        for (String ext : fileExtensionsOfFeatureSets) {
            String vectorFile = baseFileName + "." + ext + ".vec";
            MyLogger.info("Comparing " + vectorFile);
            Assert.assertTrue(TestUtils.compareTest(new File(tmpDir, vectorFile), new File(
                    dirCompare, vectorFile), "TestAudioFeatureExtraction / " + ext));
            // delete the temporary file
            new File(tmpDir, vectorFile).delete();
        }
    }

    public static void main(String[] args) throws Exception {
        new TestAudioFeatureExtraction().testFeatureExtraction();
    }
}
