package at.tuwien.ifs.feature.extraction.audio;

import java.util.ArrayList;

import org.junit.Assert;
import org.junit.Test;

import at.tuwien.ifs.feature.extraction.audio.spectrum.FFT;
import at.tuwien.ifs.somtoolbox.util.FileUtils;

/**
 * @author Rudolf Mayer
 * @version $Id: TestAudioFeatureExtractionUnits.java 160 2010-06-16 16:44:20Z mayer $
 */
public class TestAudioFeatureExtractionUnits {

    double delta = 0;// 1e-11;

    @Test
    public void testComputeMagnitude() throws Exception {
        ArrayList<String> inputDataLines = FileUtils
                .readLinesAsList(TestUtilsAudioFeatureExtraction.getResource(
                        TestUtilsAudioFeatureExtraction.RSC_MAGNITUDE + "input.txt")
                        .getAbsolutePath());
        ArrayList<String> expectedDataLines = FileUtils
                .readLinesAsList(TestUtilsAudioFeatureExtraction.getResource(
                        TestUtilsAudioFeatureExtraction.RSC_MAGNITUDE + "expected.txt")
                        .getAbsolutePath());

        for (int i = 0; i < inputDataLines.size(); i++) {
            double[] input = parseDoubleArray(inputDataLines.get(i));
            double[] expected = parseDoubleArray(expectedDataLines.get(i));
            double[] actual = new FFT().computeMagnitude(input);
            assertArrayEquals(i, expected, actual);
        }
    }

    @Test
    public void testComputePowerSpectrum() throws Exception {
        ArrayList<String> inputDataLines = FileUtils
                .readLinesAsList(TestUtilsAudioFeatureExtraction.getResource(
                        TestUtilsAudioFeatureExtraction.RSC_POWER_SPECTRUM + "input.txt")
                        .getAbsolutePath());
        ArrayList<String> expectedDataLines = FileUtils
                .readLinesAsList(TestUtilsAudioFeatureExtraction.getResource(
                        TestUtilsAudioFeatureExtraction.RSC_POWER_SPECTRUM + "expected.txt")
                        .getAbsolutePath());

        for (int i = 0; i < inputDataLines.size(); i++) {
            double[] input = parseDoubleArray(inputDataLines.get(i));
            double[] expected = parseDoubleArray(expectedDataLines.get(i));
            double[] actual = new FFT().computePowerSpectrum(input);
            assertArrayEquals(i, expected, actual);
        }
    }

    public static void main(String[] args) throws Exception {
        new TestAudioFeatureExtractionUnits().testComputePowerSpectrum();
        new TestAudioFeatureExtractionUnits().testComputePowerSpectrum();
    }

    private void assertArrayEquals(int i, double[] expectedOutput, double[] actualOutput) {
        for (int j = 0; j < actualOutput.length; j++) {
            Assert.assertEquals("Line " + (i + 1) + ", element " + (j + 1), expectedOutput[j],
                    actualOutput[j], delta);
        }
    }

    private double[] parseDoubleArray(String line) {
        String[] inputSplit = line.substring(1, line.length() - 1).split(",");
        double[] input = new double[inputSplit.length];
        for (int j = 0; j < input.length; j++) {
            input[j] = Double.parseDouble(inputSplit[j]);
        }
        return input;
    }
}
